#!/bin/perl

my %data = ();
my %mtms = (
  a => 'typ',
  b => 'min',
  c => 'max',
  x => 'typ',
  y => 'min',
  z => 'max',
);

# Copy the [Rising Waveform] and [Falling Waveform] SPICE files and
# modify to run [Composite Current] simulations.
foreach my $spi (<[abcxyz]*.spi>) {
  my $ispi = "i_$spi";
  my $ilis = $ispi;
  $ilis =~ s/spi$/lis/i;
  system("sed '/PRINT/s/V(.*)/I(VCCS2I)/i' $spi > $ispi");
  system("hspice -i $ispi -o $ilis");
}

# Collect Waveform data.
foreach my $ilis (<i_[abcxyz]*.lis>) {
  $ilis =~ /i_(.)(..)/;
  my $letter = $1;
  my $seq = $2;
  my $rf = $letter =~ /[abc]/ ? 'rise' : 'fall';
  my $mtm = $mtms{$letter};
  my $indata = 0;
  open(LIS, "<$ilis");
  while(<LIS>) {
    $indata = 1 if(/^x/);
    $indata = 0 if(/^y/);
    if($indata) {
      my ($t, $i) = split();
      if($t =~ /[0-9]/ && $i =~ /[0-9]/) {
        push(@{$data{$rf}{$seq}{$mtm}}, [ $t, (-1 * $i) ]);
      }
    }
  }
  close(LIS);
}

foreach my $rf ('rise', 'fall') {
  foreach my $seq (sort(keys(%{$data{$rf}}))) {
    open(SIM, ">${rf}_${seq}_comp_cur.sim");
    print SIM "A!Time!Node!Potential!NodeName!\n";
    open(DAT, ">${rf}_${seq}_comp_cur.dat");
    print DAT ("[Composite Current]\n");
    my @typs = @{$data{$rf}{$seq}{typ}};
    my @mins = @{$data{$rf}{$seq}{min}};
    my @maxs = @{$data{$rf}{$seq}{max}};
    foreach my $i (0..$#typs) {
      printf DAT ("%9.4fn     %9.4fm   %9.4fm   %9.4fm\n",
        $typs[$i][0] * 1e9,
	$typs[$i][1] * 1e3,
	$mins[$i][1] * 1e3,
	$maxs[$i][1] * 1e3);
      print SIM "S!$typs[$i][0]!1!$typs[$i][1]!${rf}_${seq}_typ_cur!\n";
      print SIM "S!$typs[$i][0]!1!$mins[$i][1]!${rf}_${seq}_min_cur!\n";
      print SIM "S!$typs[$i][0]!1!$maxs[$i][1]!${rf}_${seq}_max_cur!\n";
    }
    close(DAT);
    close(SIM);
  }
}
